var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React from 'react';
import { makeStyles, createStyles } from '@material-ui/styles';
import { mapToFarsi } from '../utils/farsiUtils';
import { formatDateString, formatDateTimeString } from '../utils/dateUtils';
import { format as formatter } from "d3-format";
var useStyles = makeStyles(function (theme) { return createStyles({
    label: {
        fontSize: '0.8em',
        color: theme.palette.grey[700],
    },
    text: {},
    textImportant: {
        fontWeight: 700,
    },
    inline: {
        display: 'inline-block',
    },
    ltr: {
        flip: false,
        direction: 'ltr',
    },
}); });
var DataView = function (props) {
    var value = props.value, label = props.label, format = props.format, scale = props.scale, important = props.important, className = props.className, inline = props.inline, span = props.span;
    var classes = useStyles();
    var formattedValue = formatValue(value, format, scale);
    if (span) {
        return React.createElement("span", null,
            label ? label + ": " : '',
            React.createElement("span", { className: (formattedValue.ltr ? ' ' + classes.ltr : '') }, formattedValue.formattedValue));
    }
    else {
        var classNames = inline || className ? { className: (className ? className : '') + (!inline ? '' : ' ' + classes.inline) } : undefined;
        return (React.createElement("div", __assign({}, classNames),
            label && React.createElement("span", { className: classes.label },
                label,
                " : "),
            React.createElement("span", { className: (important ? classes.textImportant : classes.text) + ' ' + (formattedValue.ltr ? classes.ltr : '') }, formattedValue.formattedValue)));
    }
};
function formatValue(value, format, scale) {
    var formattedValue = '';
    var ltr = false;
    if (!value && value !== 0)
        return { formattedValue: formattedValue, ltr: ltr };
    if (format === 'text') {
        formattedValue = value;
    }
    else if (format === 'date') {
        formattedValue = mapToFarsi(formatDateString(value));
        ltr = true;
    }
    else if (format === 'timestamp') {
        formattedValue = mapToFarsi(formatDateTimeString(value));
        ltr = true;
    }
    else if (format === 'number') {
        formattedValue = mapToFarsi(value.toString());
        ltr = true;
    }
    else if (format === 'decimal') {
        var decimalFormatter = getFormatter(scale);
        formattedValue = mapToFarsi(decimalFormatter(+value));
        ltr = true;
    }
    else {
        formattedValue = 'unknown format';
    }
    if (!formattedValue)
        formattedValue = '';
    return { formattedValue: formattedValue, ltr: ltr };
}
var formatters = {};
function getFormatter(scale) {
    if (!scale) {
        scale = 0;
    }
    var f = formatters[scale];
    if (!f) {
        f = formatter("(,." + scale + "f");
        formatters[scale] = f;
    }
    return f;
}
export default DataView;
