var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useEffect, useContext, useLayoutEffect, useRef, useState } from 'react';
import { useHistory, useLocation } from 'react-router';
import { observer } from 'mobx-react-lite';
import { makeStyles, createStyles } from '@material-ui/core';
import Container from '@material-ui/core/Container';
import Divider from '@material-ui/core/Divider';
import Button from '@material-ui/core/Button';
import Grid from '@material-ui/core/Grid';
import CircularProgress from '@material-ui/core/CircularProgress';
import RefreshIcon from '@material-ui/icons/Refresh';
import ArrowBack from '@material-ui/icons/ArrowForward';
import AdvanceIcon from '@material-ui/icons/MoreHoriz';
import FilterIcon from '@material-ui/icons/FilterList';
import PrintIcon from '@material-ui/icons/Print';
import { AuthStoreContext } from '../store/authStore';
import { AppStoreContext } from '../store/appStore';
import useKamandData from '../hooks/useKamandData';
import { useSetFilter, createQueryString } from '../hooks/useSetFilters';
import { useTranslation } from 'react-i18next';
import { useWindowSize } from '../hooks/useWindowSize';
import { calcTotalOffset } from '../utils/generalUtils';
import ReactToPrint from 'react-to-print';
import './print.css';
var useStyles = makeStyles(function (theme) {
    var _a;
    var _b;
    return createStyles((_a = {
            root: {
            // marginBottom: '45px',
            },
            btnContainer: {
                margin: 0,
                marginTop: 40,
            },
            backBtn: {
                display: 'flex',
                justifyContent: 'flex-start',
            },
            page: {
                flip: false,
                padding: 0,
                direction: theme.direction
            },
            table: {
                minWidth: 700,
            },
            tableScroll: {
                overflow: 'auto',
                maxHeight: 300
            },
            rowRoot: {
                marginTop: '5px',
            },
            selectedRow: {
                backgroundColor: ((_b = theme.palette.selected) === null || _b === void 0 ? void 0 : _b.row) || theme.palette.grey[300],
            },
            cellHiddenPrint: {
                visibility: 'visible',
            },
            headerCell1: {
                position: 'sticky',
                zIndex: 1,
                top: 0,
                // height: '1em',
                textAlign: 'center',
                backgroundColor: theme.palette.background.default,
            },
            headerCell2: {
                position: 'sticky',
                zIndex: 1,
                top: '2.6em',
                // height: '1em',
                textAlign: 'center',
                backgroundColor: theme.palette.background.default,
            },
            summaryCell3: {
                position: 'sticky',
                zIndex: 1,
                top: 'calc(100% - 2.6em)',
                textAlign: 'center',
                backgroundColor: theme.palette.primary.light,
            },
            summarySpaceHolder: {
                height: '2.6em',
            },
            invisible: {
                visibility: 'hidden',
            },
            cellClickable: {
                cursor: 'pointer',
            },
            cellCode: {
                flip: false,
                textAlign: 'left',
            },
            cellNumber: {
                flip: false,
                textAlign: 'right',
            }
        },
        _a['@media screen'] = {
            table: {
                minWidth: 1200,
            },
            cellHiddenPrint: {},
        },
        _a['@media print'] = {
            table: {
                maxWidth: '100%',
                width: '100%',
            },
            tableScroll: {
                overflow: 'unset',
                maxHeight: 'unset',
            },
            '@global': {
                html: {
                // fontSize: '10px',
                },
            },
            cellHiddenPrint: {
                display: 'none',
            },
            page: {
                padding: 20,
            },
        },
        _a));
});
export var provideFilterKeys = function (filters) { return filters.filter(function (f) { return f.source === 'app'; }).map(function (f) { return f.key; }); };
var ReportFilters = observer(function (props) {
    var t = useTranslation().t;
    var filters = props.filters, classes = props.classes, showReport = props.showReport;
    var history = useHistory();
    var appStore = useContext(AppStoreContext);
    var _a = useState(null), advanceMode = _a[0], setAdvanceMode = _a[1];
    useEffect(function () {
        if (advanceMode === null) {
            var hasAdvancedFilter = false;
            for (var i = 0; i < filters.length; i++) {
                if (!filters[i].advanced)
                    continue;
                if (appStore.getFilter(filters[i].key)) {
                    hasAdvancedFilter = true;
                    break;
                }
            }
            setAdvanceMode(hasAdvancedFilter);
        }
    }, [advanceMode, filters, appStore]);
    var handleGoBack = function () {
        if (history.length > 0) {
            //TODO we have to check if the back url is in reports
            history.goBack();
        }
        else {
            history.push('/reports');
        }
    };
    var handleAdvance = function () {
        setAdvanceMode(!advanceMode);
    };
    return (React.createElement(Container, { className: classes.root },
        React.createElement(Divider, null),
        React.createElement(Grid, { container: true, spacing: 1, direction: "row", justify: "flex-start", alignItems: "flex-end" },
            React.createElement(Grid, { item: true, xs: 12 },
                React.createElement(Grid, { container: true, direction: "row", justify: "flex-start" },
                    React.createElement(Grid, { item: true },
                        React.createElement(Button, { onClick: handleGoBack },
                            React.createElement(ArrowBack, null))),
                    React.createElement(Grid, { item: true },
                        React.createElement(Button, { onClick: handleAdvance },
                            React.createElement(AdvanceIcon, null))))),
            filters.filter(function (f) { return f.editor && (advanceMode || !f.advanced); }).map(function (f) { return React.createElement(Filter, { key: f.key, filter: f }); }),
            React.createElement(Grid, { item: true, xs: 12, container: true, spacing: 1, justify: "center", className: classes.btnContainer },
                React.createElement(Grid, { item: true, xs: 4, sm: 3 },
                    React.createElement(Button, { variant: "contained", color: "secondary", onClick: showReport, fullWidth: true }, t('pbl.executeReport')))))));
});
var Filter = function (_a) {
    var filter = _a.filter;
    var t = useTranslation().t;
    var appStore = useContext(AppStoreContext);
    var key = filter.key, label = filter.label, editor = filter.editor;
    var _b = useState(''), value = _b[0], setValue = _b[1];
    var latestValue = useRef(null);
    var handleChange = function (e) {
        setValue(e.target.value);
        appStore.setFilter(key, e.target.value);
    };
    var newValue = appStore.getFilter(key);
    useEffect(function () {
        if (latestValue.current !== newValue) {
            if (newValue !== null && newValue !== undefined) {
                setValue(newValue);
            }
        }
        latestValue.current = newValue;
    }, [newValue, latestValue]);
    if (!editor)
        return null; // K1 this line never runs, as the filter is checked 
    var EditorComponent = editor.EditorComponent;
    return React.createElement(EditorComponent, { key: key, label: t(label), value: value, handleChange: handleChange });
};
var dataOptions = function (query, filters, appStore, authStore) {
    var valueOfFilter = (function (f) {
        if (f.source === 'app')
            return appStore.getFilter(f.key);
        if (f.source === 'auth')
            return authStore.getOptionData(f.key);
    });
    return {
        query: query,
        queryParams: function () { return filters.reduce(function (r, f) {
            var _a;
            return (__assign(__assign({}, r), (_a = {}, _a[f.key] = valueOfFilter(f), _a)));
        }, {}); },
        publicQuery: false,
        notReady: function (queryParams) { return filters.filter(function (f) { return f.mandatory; }).reduce(function (r, f) { return (r || !queryParams[f.key]); }, false); },
    };
};
var FilteredReport = observer(function (props) {
    var t = useTranslation().t;
    var query = props.query, title = props.title, filters = props.filters, TableComponent = props.TableComponent, HeaderComponent = props.HeaderComponent;
    var classes = useStyles();
    var appStore = useContext(AppStoreContext);
    var authStore = useContext(AuthStoreContext);
    var history = useHistory();
    var location = useLocation();
    var tab = location.hash ? location.hash.substring(1) : 'F';
    useEffect(function () {
        appStore.setPageTitle(title);
        appStore.setOptionData('hideFab', true);
        appStore.hideAppBar(false);
    }, [appStore, title]);
    var filterKeys = provideFilterKeys(filters);
    var filtersReady = useSetFilter(filterKeys || []);
    var showReport = function () {
        // history.push(`#R`);
        var options = dataOptions(query, filters, appStore, authStore);
        var queryParams;
        if (typeof options.queryParams === 'function') {
            queryParams = options.queryParams();
        }
        else {
            queryParams = options.queryParams;
        }
        var r = filters.filter(function (f) { return f.mandatory; }).map(function (f) {
            var r = !!queryParams[f.key];
            if (!r)
                appStore.setSnackMessage({ message: t('error.completeField', { field: f.label }) });
            return r;
        }).reduce(function (r, f) { return (r && f); }, true);
        if (r) {
            var params = filters.map(function (f) { return ({ key: f.key, value: appStore.getFilter(f.key) }); });
            var queryString = createQueryString(params);
            var newLocation = "" + location.pathname + queryString + "#R";
            history.push(newLocation);
        }
    };
    var handleGoBack = function () {
        if (history.length > 0) {
            //TODO we have to check if the back url is in reports
            history.goBack();
        }
        else {
            history.push("" + location.pathname + location.search + "#F");
        }
    };
    var handleShowFilter = function () {
        history.push("" + location.pathname + location.search + "#F");
    };
    return (React.createElement(React.Fragment, null,
        filtersReady && tab === 'F' && React.createElement(ReportFilters, { filters: filters, classes: classes, showReport: showReport }),
        filtersReady && tab === 'R' && React.createElement(ReportTable, { query: query, filterKeys: filterKeys, filters: filters, HeaderComponent: HeaderComponent, TableComponent: TableComponent, classes: classes, handleGoBack: handleGoBack, handleShowFilter: handleShowFilter })));
});
var ReportTable = observer(function (props) {
    var query = props.query, filterKeys = props.filterKeys, filters = props.filters, TableComponent = props.TableComponent, HeaderComponent = props.HeaderComponent, classes = props.classes, handleGoBack = props.handleGoBack, handleShowFilter = props.handleShowFilter;
    var appStore = useContext(AppStoreContext);
    var authStore = useContext(AuthStoreContext);
    var size = useWindowSize();
    var tableDivRef = useRef(null);
    useLayoutEffect(function () {
        if (tableDivRef.current) {
            var offsetTop = calcTotalOffset(tableDivRef.current);
            var maxHeight = size.h - offsetTop - 20;
            tableDivRef.current.style.maxHeight = maxHeight + "px";
        }
    });
    var _a = useKamandData(dataOptions(query, filters, appStore, authStore)), queryData = _a.queryData, refreshHandler = _a.refreshHandler;
    var componentRef = useRef(null);
    return (React.createElement(Container, { className: classes.root },
        React.createElement(Grid, { container: true, spacing: 1, direction: "row", justify: "flex-start" },
            React.createElement(Grid, { item: true },
                React.createElement(Button, { onClick: handleGoBack, fullWidth: true, className: classes.backBtn },
                    React.createElement(ArrowBack, null))),
            queryData && queryData.loading &&
                React.createElement(Grid, { item: true },
                    React.createElement(CircularProgress, null)),
            (!queryData || !queryData.loading) &&
                React.createElement(Grid, { item: true },
                    React.createElement(Button, { onClick: handleShowFilter },
                        React.createElement(FilterIcon, null)),
                    React.createElement(Button, { onClick: refreshHandler },
                        React.createElement(RefreshIcon, null)),
                    React.createElement(ReactToPrint, { trigger: function () { return React.createElement(Button, null,
                            React.createElement(PrintIcon, null)); }, content: function () { return componentRef.current; } }))),
        queryData && queryData.error && React.createElement("p", null, "error"),
        React.createElement("div", { ref: componentRef, className: classes.page },
            queryData && React.createElement(HeaderComponent, { queryParams: queryData.queryParams, classes: classes }),
            queryData && !queryData.loading && !queryData.error && queryData.data && (React.createElement("div", { ref: tableDivRef, className: classes.tableScroll },
                React.createElement(TableComponent, { queryParams: queryData.queryParams, data: queryData.data, selection: queryData.selection, classes: classes, filterKeys: filterKeys }))))));
});
export default (FilteredReport);
export var makeReportUrl = function (baseUser, filterKeys, argParams, queryParams) {
    var params = filterKeys.map(function (f) { return ({ key: f, value: (argParams[f] || queryParams[f]) }); });
    var queryString = createQueryString(params);
    var url = "" + baseUser + queryString + "#R";
    return url;
};
