import React from 'react';
export interface FormValues {
    [field: string]: any;
}
export declare type FormErrors<Values> = {
    [K in keyof Values]?: Values[K] extends any[] ? Values[K][number] extends object ? FormErrors<Values[K][number]>[] | string | string[] : string | string[] : Values[K] extends object ? FormErrors<Values[K]> : string;
};
export declare type FormTouched<Values> = {
    [K in keyof Values]?: Values[K] extends any[] ? Values[K][number] extends object ? FormTouched<Values[K][number]>[] : boolean : Values[K] extends object ? FormTouched<Values[K]> : boolean;
};
export interface FormState<Values> {
    values?: Values;
    errors: FormErrors<Values>;
    touched: FormTouched<Values>;
    isSubmitting: boolean;
    isValidating: boolean;
    submitCount: number;
}
export interface FormSubmitResult<Values> {
    errors?: FormErrors<Values>;
    valuesFromServer?: Values;
}
export interface FormConfig<Values> {
    initialValues?: Values;
    validate?: (values: Values) => Promise<FormErrors<Values>>;
    submit: (values: Values, form: KamandForm<Values>) => Promise<FormSubmitResult<Values>>;
    onChange?: (path: string, previousValue: any, value: any) => void | Promise<void>;
}
export interface FormFieldProps {
    onChange: (e: React.ChangeEvent<any>) => void;
    name: string;
    id: string;
    value: any;
    error: boolean;
    helperText: Node;
}
export interface KamandForm<Values> {
    getFieldProps: (path: string) => FormFieldProps;
    values: Values;
    isDirty: () => boolean;
    isSubmitting: () => boolean;
    isValidating: () => boolean;
    submitForm: () => Promise<boolean>;
    resetForm: () => void;
    setFieldValue: (path: string, value: any) => void;
    removeFromArray: (pathToArray: string, index: number) => void;
    getHelperText(path: string): Node;
}
export declare function useKamandForm<Values extends FormValues = FormValues>(props: FormConfig<Values>): KamandForm<Values>;
