import { Context } from 'react';
import { ISnackMessage } from './interfaces/authInterfaces';
import { IQueryParams, ISelection } from './interfaces/dataInterfaces';
export declare class AppStore {
    pageTitle?: string;
    appBarHidden: boolean;
    snackMessage?: ISnackMessage;
    readonly queryData: import("mobx").ObservableMap<string, any>;
    readonly paginatedQueryData: import("mobx").ObservableMap<string, PaginatedQueryData>;
    readonly filtersData: import("mobx").ObservableMap<string, any>;
    readonly actData: import("mobx").ObservableMap<string, any>;
    readonly optionData: import("mobx").ObservableMap<string, any>;
    constructor();
    setPageTitle(pageTitle: string): void;
    hideAppBar(appBarHidden: boolean): void;
    setSnackMessage(snackMessage?: ISnackMessage): void;
    setFilter(key: string, filter?: any): void;
    getFilter(key: string): any;
    setQueryData(key: string, queryParams: IQueryParams, data: any[] | undefined, loading: boolean, error: boolean): void;
    getQueryData(key: string): any;
    invalidateQueryDataCache(key: string): any;
    prepareQueryData(key: string, query: string, queryParams: IQueryParams, forceRefresh: boolean, publicQuery?: boolean, makeupData?: ((data: any) => any)): Promise<void>;
    getPaginatedQueryData(key: string): PaginatedQueryData | undefined;
    invalidatePaginatedQueryDataCache(key: string): void;
    preparePaginatedQueryData(key: string, query: string, queryParams: IQueryParams, forceRefresh: boolean, publicQuery?: boolean, pageSize?: number, makeupData?: ((data: any) => any)): void;
    setActData(key: string, actData?: any): void;
    getActData(key: string): any;
    deleteActData(key: string): any;
    loadActDataFirstCache(key: string, query: string, queryParams: any, makeObservable?: (data: any) => any): Promise<any>;
    private loadingActData;
    loadActData(key: string, query: string, queryParams: any, makeObservable?: (data: any) => any): Promise<any>;
    saveActData(key: string | null, query: string, data: any): Promise<any>;
    removeActData(key: string | null, query: string, data: any): Promise<any>;
    doAsyncActData(query: string, data: any): void;
    listenAsyncActData(event: string, callback: ((payload: any) => void)): void;
    setOptionData(key: string, data: any): void;
    getOptionData(key: string): any;
}
export declare const appStore: AppStore;
export declare const AppStoreContext: Context<AppStore>;
export declare class Selection implements ISelection {
    private selection;
    constructor();
    isSelected(index: number): boolean;
    setSelected(index: number, selected: boolean): void;
    get selected(): boolean;
}
export declare class PaginatedQueryData {
    private query;
    private queryParams;
    private publicQuery;
    private pageSize;
    private makeupData;
    readonly pages: import("mobx").IObservableArray<PageData | null>;
    totalCount: number;
    activePageIndex: number;
    constructor(query: string, queryParams: IQueryParams, publicQuery: boolean, pageSize: number, makeupData: ((data: any) => any) | undefined);
    getActivePageIndex(): number;
    setActivePageIndex(value: number): void;
    get pageCount(): number;
    setMeta(totalCount: number, pageSize: number): void;
    getPage(index: number): PageData | undefined;
    get currentPage(): PageData | undefined;
    preparePageData(index: number): Promise<void>;
}
export declare class PageData {
    private pageIndex;
    data?: any[];
    loading: boolean;
    error: boolean;
    constructor(pageIndex: number);
    setData(data: any[]): void;
    setError(): void;
    getIndex(): number;
    getData(): any[] | undefined;
}
